/****************************************************************************
 Copyright (c) 2010-2013 Freescale Semiconductor, Inc.
 ALL RIGHTS RESERVED.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the <organization> nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 ***************************************************************************
 *
 * This code is derived from the Freescale SDK
 * It assumes PSECTOR_SIZE, PBLOCK_SIZE and PBLOCK_BASE are defined in config.h
 *
 ***************************************************************************
 */
#include "config.h"
#if INCLUDE_FLASH
#include "MK60D10.h"
#include "flash.h"

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

typedef unsigned char BOOL;
typedef void (* PCALLBACK)(void);

typedef signed char INT8;
typedef unsigned char UINT8;
typedef volatile signed char VINT8;
typedef volatile unsigned char VUINT8;

typedef signed short INT16;
typedef unsigned short UINT16;
typedef volatile signed short VINT16;
typedef volatile unsigned short VUINT16;

typedef signed long INT32;
typedef unsigned long UINT32;
typedef volatile signed long VINT32;
typedef volatile unsigned long VUINT32;

#define WORD2BYTE(a)					a
#define ENTER_DEBUG_MODE				asm ("BKPT #0" )
#define GET_BIT_0_7(value)              ((UINT8)((value) & 0xFF))
#define GET_BIT_8_15(value)             ((UINT8)(((value)>>8) & 0xFF))
#define GET_BIT_16_23(value)            ((UINT8)(((value)>>16) & 0xFF))
#define GET_BIT_24_31(value)            ((UINT8)((value)>>24))
#define REG_BIT_SET(address, mask)      (*(VUINT8*)(address) |= (mask))
#define REG_BIT_CLEAR(address, mask)    (*(VUINT8*)(address) &= ~(mask))
#define REG_BIT_GET(address, mask)      (*(VUINT8 *)(address) & (UINT8)(mask))
#define REG_WRITE(address, value)       (*(VUINT8*)(address) = (value))
#define REG_READ(address)               ((UINT8)(*(VUINT8*)(address)))
#define REG_WRITE32(address, value)     (*(VUINT32*)(address) = (value))
#define REG_READ32(address)             ((UINT32)(*(VUINT32*)(address)))
#define NULL_CALLBACK                   ((PCALLBACK)0xFFFFFFFF)


typedef struct _ssd_config
{
    UINT32      ftfxRegBase;        /* FTFx control register base */
    UINT32      PFlashBlockBase;    /* base address of PFlash block */
    UINT32      PFlashBlockSize;    /* size of PFlash block */
    UINT32      DFlashBlockBase;    /* base address of DFlash block */
    UINT32      DFlashBlockSize;    /* size of DFlash block */
    UINT32      EERAMBlockBase;     /* base address of EERAM block */
    UINT32      EEEBlockSize;       /* size of EEE block */
    BOOL        DebugEnable;        /* background debug mode enable bit */
    PCALLBACK   CallBack;           /* pointer to callback function */
} FLASH_SSD_CONFIG, *PFLASH_SSD_CONFIG;

typedef UINT32 (*pFLASHCOMMANDSEQUENCE) (PFLASH_SSD_CONFIG PSSDConfig);

#define FTFx_OK                            0x0000
#define FTFx_ERR_MGSTAT0                   0x0001
#define FTFx_ERR_PVIOL                     0x0010
#define FTFx_ERR_ACCERR                    0x0020
#define FTFx_ERR_CHANGEPROT                0x0100
#define FTFx_ERR_NOEEE                     0x0200
#define FTFx_ERR_EFLASHONLY                0x0400
#define FTFx_ERR_RAMRDY                    0x0800
#define FTFx_ERR_RANGE                     0x1000
#define FTFx_ERR_SIZE                      0x2000

/* Flash Status Register (FSTAT)*/
#define FTFx_SSD_FSTAT_OFFSET               0x00000000
/* Flash configuration register (FCNFG)*/
#define FTFx_SSD_FCNFG_OFFSET               0x00000001
/* Flash security register (FSEC) */
#define FTFx_SSD_FSEC_OFFSET                0x00000002
/* Flash Option Register (FOPT) */
#define FTFx_SSD_FOPT_OFFSET                0x00000003
/* Flash common command object registers (FCCOB0-B) */
#define FTFx_SSD_FCCOB0_OFFSET              0x00000007
#define FTFx_SSD_FCCOB1_OFFSET              0x00000006
#define FTFx_SSD_FCCOB2_OFFSET              0x00000005
#define FTFx_SSD_FCCOB3_OFFSET              0x00000004
#define FTFx_SSD_FCCOB4_OFFSET              0x0000000B
#define FTFx_SSD_FCCOB5_OFFSET              0x0000000A
#define FTFx_SSD_FCCOB6_OFFSET              0x00000009
#define FTFx_SSD_FCCOB7_OFFSET              0x00000008
#define FTFx_SSD_FCCOB8_OFFSET              0x0000000F
#define FTFx_SSD_FCCOB9_OFFSET              0x0000000E
#define FTFx_SSD_FCCOBA_OFFSET              0x0000000D
#define FTFx_SSD_FCCOBB_OFFSET              0x0000000C
/* P-Flash protection registers (FPROT0-3) */
#define FTFx_SSD_FPROT0_OFFSET              0x00000013
#define FTFx_SSD_FPROT1_OFFSET              0x00000012
#define FTFx_SSD_FPROT2_OFFSET              0x00000011
#define FTFx_SSD_FPROT3_OFFSET              0x00000010
/* D-Flash protection registers (FDPROT) */
#define FTFx_SSD_FDPROT_OFFSET              0x00000017
/* EERAM Protection Register (FEPROT)  */
#define FTFx_SSD_FEPROT_OFFSET              0x00000016

#define FTFx_SSD_FSTAT_CCIF                 0x80
#define FTFx_SSD_FSTAT_RDCOLERR             0x40
#define FTFx_SSD_FSTAT_ACCERR               0x20
#define FTFx_SSD_FSTAT_FPVIOL               0x10
#define FTFx_SSD_FSTAT_MGSTAT0              0x01
#define FTFx_SSD_FSTAT_ERROR_BITS           (FTFx_SSD_FSTAT_ACCERR \
                                            |FTFx_SSD_FSTAT_FPVIOL \
                                            |FTFx_SSD_FSTAT_MGSTAT0)

/*------------- Flash hardware algorithm operation commands -------------*/
#define FTFx_VERIFY_BLOCK               0x00
#define FTFx_VERIFY_SECTION             0x01
#define FTFx_PROGRAM_CHECK              0x02
#define FTFx_READ_RESOURCE              0x03
#define FTFx_PROGRAM_LONGWORD           0x06
#define FTFx_PROGRAM_PHRASE             0x07
#define FTFx_ERASE_BLOCK                0x08
#define FTFx_ERASE_SECTOR               0x09
#define FTFx_PROGRAM_SECTION            0x0B
#define FTFx_VERIFY_ALL_BLOCK           0x40
#define FTFx_READ_ONCE                  0x41
#define FTFx_PROGRAM_ONCE               0x43
#define FTFx_ERASE_ALL_BLOCK            0x44
#define FTFx_SECURITY_BY_PASS           0x45
#define FTFx_PFLASH_SWAP                0x46
#define FTFx_PROGRAM_PARTITION          0x80
#define FTFx_SET_EERAM                  0x81

/* PFlash swap control codes */
#define FTFx_SWAP_SET_INDICATOR_ADDR    0x01
#define FTFx_SWAP_SET_IN_PREPARE        0x02
#define FTFx_SWAP_SET_IN_COMPLETE       0x04
#define FTFx_SWAP_REPORT_STATUS         0x08


/* EERAM Function Control Code */
#define EEE_ENABLE                      0x00
#define EEE_DISABLE                     0xFF

/* PFlash swap states */
#define FTFx_SWAP_UNINIT                0x00
#define FTFx_SWAP_READY                 0x01
#define FTFx_SWAP_UPDATE                0x02
#define FTFx_SWAP_UPDATE_ERASED         0x03
#define FTFx_SWAP_COMPLETE              0x04

#define PGM_SIZE_BYTE					4

#include "config.h"
#include "flash.h"
#include <stdio.h>
#include <string.h>

#define DUMMY_FLASHID	99
#define SECTORTOT		(PBLOCK_SIZE/PSECTOR_SIZE)

FLASH_SSD_CONFIG flashSSDConfig = {
    0x40020000, PBLOCK_BASE, PBLOCK_SIZE, 0xffffffff, 0, 0xffffffff, 0, 0, NULL_CALLBACK
};

long k60_erase_buf[64];
long k60_write_buf[96];
#if INCLUDE_FLASHEWRITE
long k60_ewrite_buf[160];
#endif

/* k60_erase():
 * This is a hack from the SDK function FlashEraseSector to fit
 * the uMon-M flash model.  This function assumes it resides in ram.
 */
int
k60_erase(struct flashinfo *fdev, int snum)
{
	UINT32 dest;

	if (snum >= SECTORTOT)
		return(-1);

	dest = snum * PSECTOR_SIZE; 
     
	/* clear RDCOLERR & ACCERR & FPVIOL flag in flash status register. Write 1 to clear*/
	REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET,FTFx_SSD_FSTAT_ERROR_BITS);   

	/* passing parameter to the command */
	REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB0_OFFSET, FTFx_ERASE_SECTOR);
	REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB1_OFFSET, GET_BIT_16_23(dest));
	REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB2_OFFSET, GET_BIT_8_15(dest));
	REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB3_OFFSET, GET_BIT_0_7(dest));

	/* clear CCIF to launch command */
	REG_BIT_SET(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, FTFx_SSD_FSTAT_CCIF); 

	/* wait for completion of this command */
	while((REG_BIT_GET(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, FTFx_SSD_FSTAT_CCIF)) == 0);
            
	if ((REG_READ(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET) & FTFx_SSD_FSTAT_ERROR_BITS) != 0)
		return(-2);

	return(0);
}
void
k60_erase_end(void)
{}

int
k60_write(struct flashinfo *fdev,unsigned char *_dest,unsigned char *pData,long size)
{
    UINT8 i;
    UINT32 temp, dest;
	char tmpsrc[PGM_SIZE_BYTE];

	dest = (UINT32)_dest;
    
    /* convert to byte addr */
	temp = flashSSDConfig.PFlashBlockBase;
	if((dest >= temp) && (dest < (temp + flashSSDConfig.PFlashBlockSize)))    {
		dest -= temp;
	} else {
		return(-2);
	}
    while(size > 0)
    {
		if (size < PGM_SIZE_BYTE) {
			for(i=0;i<PGM_SIZE_BYTE;i++)
				tmpsrc[i] = ((char *)dest)[i];
			for(i=0;i<size;i++)
				tmpsrc[i] = pData[i];
			pData = (unsigned char *)tmpsrc;
		}

        /* clear RDCOLERR & ACCERR & FPVIOL flag in flash status register. Write 1 to clear*/
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET,FTFx_SSD_FSTAT_ERROR_BITS);   
		/* passing parameter to the command */
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB0_OFFSET, FTFx_PROGRAM_LONGWORD);
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB1_OFFSET, GET_BIT_16_23(dest));
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB2_OFFSET, GET_BIT_8_15(dest));
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB3_OFFSET, GET_BIT_0_7(dest));
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB4_OFFSET, pData[3]);
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB5_OFFSET, pData[2]);
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB6_OFFSET, pData[1]);
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB7_OFFSET, pData[0]);
        
		/* clear CCIF to launch command */
		REG_BIT_SET(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, FTFx_SSD_FSTAT_CCIF); 

		/* wait for completion of this command */
		while((REG_BIT_GET(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, FTFx_SSD_FSTAT_CCIF)) == 0);
            
		if ((REG_READ(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET) & FTFx_SSD_FSTAT_ERROR_BITS) != 0)
			return(-3);
            
		// update source, destination and count:
		dest += PGM_SIZE_BYTE;
		size -= PGM_SIZE_BYTE;
		pData += PGM_SIZE_BYTE;
    }
    return(0);
}

void
k60_write_end(void)
{}

/* k60_ewrite():
 * This is essentially a combination of erase and write all in one
 * function.
 */
int
k60_ewrite(struct flashinfo *fdev,unsigned char *dest,unsigned char *src,long bytecnt)
{
#if INCLUDE_FLASHEWRITE
	int i;
	UINT32 addr;
	UINT8 tmpsrc[PGM_SIZE_BYTE];

	/****** CHECK:
	 * First we make sure that the address is within range of our flash configuration:
	 * If not, just return -1 here.
	 */
	if (((int)dest < flashSSDConfig.PFlashBlockBase) || 
		((int)(dest+bytecnt) >= (flashSSDConfig.PFlashBlockBase + flashSSDConfig.PFlashBlockSize)))
		return(-1);

	/****** ERASE:
	 * Apply the flash erase algorithm to all sectors within the incoming range.
	 */
	addr = (UINT32)(fdev->base);

	/* For each sector, if it overlaps any of the destination space
	 * then erase that sector.
	 */
	for (i=0;i<fdev->sectorcnt;i++) {
		if ((((UINT8 *)dest) > (fdev->sectors[i].end)) ||
			(((UINT8 *)dest+bytecnt-1) < (fdev->sectors[i].begin))) {
			addr += fdev->sectors[i].size;
			continue;
		}

		/* clear RDCOLERR & ACCERR & FPVIOL flag in flash status register. Write 1 to clear*/
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET,FTFx_SSD_FSTAT_ERROR_BITS);   

		/* passing parameter to the command */
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB0_OFFSET, FTFx_ERASE_SECTOR);
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB1_OFFSET, GET_BIT_16_23(addr));
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB2_OFFSET, GET_BIT_8_15(addr));
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB3_OFFSET, GET_BIT_0_7(addr));

		/* clear CCIF to launch command */
		REG_BIT_SET(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, FTFx_SSD_FSTAT_CCIF); 

		/* wait for completion of this command */
		while((REG_BIT_GET(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, FTFx_SSD_FSTAT_CCIF)) == 0);
            
		if ((REG_READ(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET) & FTFx_SSD_FSTAT_ERROR_BITS) != 0)
			return(-2);

        addr += fdev->sectors[i].size;
	}

	/****** WRITE:
	 * At this point, all the concerned sections have been erased, now we
	 * begin the reprogramming...
	 */

	// Adjust the address to be relative to the base of the flash space.
	addr = (UINT32)(dest - flashSSDConfig.PFlashBlockBase);

    while(bytecnt > 0) {

		if (bytecnt < PGM_SIZE_BYTE) {
			for(i=0;i<PGM_SIZE_BYTE;i++)
				tmpsrc[i] = ((char *)addr)[i];
			for(i=0;i<bytecnt;i++)
				tmpsrc[i] = src[i];
			src = (unsigned char *)tmpsrc;
		}

		// clear RDCOLERR & ACCERR & FPVIOL flag in flash status register. Write 1 to clear
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET,FTFx_SSD_FSTAT_ERROR_BITS);   

		// pass parameter to the command
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB0_OFFSET, FTFx_PROGRAM_LONGWORD);
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB1_OFFSET, GET_BIT_16_23(addr));
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB2_OFFSET, GET_BIT_8_15(addr));
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB3_OFFSET, GET_BIT_0_7(addr));
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB4_OFFSET, src[3]);
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB5_OFFSET, src[2]);
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB6_OFFSET, src[1]);
		REG_WRITE(flashSSDConfig.ftfxRegBase + FTFx_SSD_FCCOB7_OFFSET, src[0]);
        
		// clear CCIF to launch command
		REG_BIT_SET(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, FTFx_SSD_FSTAT_CCIF); 

		// wait for completion of this command
		while((REG_BIT_GET(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET, FTFx_SSD_FSTAT_CCIF)) == 0);
            
		if ((REG_READ(flashSSDConfig.ftfxRegBase + FTFx_SSD_FSTAT_OFFSET) & FTFx_SSD_FSTAT_ERROR_BITS) != 0)
			return(-3);
            
		// update source, destination and count:
		src += PGM_SIZE_BYTE;
		addr += PGM_SIZE_BYTE;
		bytecnt -= PGM_SIZE_BYTE;
    }
	
	/****** RESET:
	 * At this point (hopefully) everything has been rewritten, so trigger the
	 * watchdog to cause a reset...
	 */
	WDOG_UNLOCK = 0xC520U;
	WDOG_UNLOCK = 0xD928U;
	WDOG_TOVALH = 0U;
	WDOG_TOVALL = 0x100U;
	WDOG_STCTRLH = 0x5f9U;
    while(1);

	return(0);	// Won't get here
#else
	printf("ewrite not supported\n");
	return(-1);
#endif
}

void
k60_ewrite_end(void)
{}


int
k60_type(struct flashinfo *fdev)
{
	fdev->id = DUMMY_FLASHID;
	return(fdev->id);
}

int
k60_lock(struct flashinfo *fdev,int snum,int operation)
{
    if (operation == FLASH_LOCKABLE)
        return(0);
    return(-1);
}

struct flashdesc FlashNamId[] = {
	{ DUMMY_FLASHID,	"K6X-Flash"}
};

struct sectorinfo sinfoK60[SECTORTOT];

/* FlashBankInit():
 * Initialize flash structures and determine flash device type.
 */
int
FlashBankInit(struct flashinfo *fbnk, int snum)
{
	int	i, ssize;
	unsigned long addr;

	addr = (unsigned long)(fbnk->base);
	fbnk->sectorcnt = SECTORTOT;
	ssize = PSECTOR_SIZE;

	for(i=0;i<SECTORTOT;i++) {
		fbnk->sectors[i].snum = snum+i;
		fbnk->sectors[i].size = ssize;
		fbnk->sectors[i].begin = (unsigned char *)addr;
		fbnk->sectors[i].end = (unsigned char *)(addr + ssize - 1);
		addr += ssize;
	}
	fbnk->end = (unsigned char *)(addr - 1);
	return(SECTORTOT);
}

#define BxDCE			(1<<4)

/* FlashInit():
 * Initialize data structures for each bank of flash...
 */
int
FlashInit(void)
{
	struct	flashinfo *fbnk;
	unsigned long from, to;
	int		err, snum, fsize;

	err = snum = 0;
	FlashCurrentBank = 0;

	/* Disable d-cache on flash... */
	FMC_PFB0CR &= ~BxDCE;
	FMC_PFB1CR &= ~BxDCE;

	fbnk = &FlashBank[0];
	k60_type(fbnk);
	fbnk->base = (unsigned char *)FLASH_BANK0_BASE_ADDR;
	fbnk->width = 0;	/* Not needed for this device. */

	/* Relocate (to ram) the functions that operate on flash. */
	/* ERASE: */
	from = (unsigned long)(k60_erase) - 1;
	fsize = (unsigned long)(k60_erase_end) - (unsigned long)(k60_erase);
	to = (unsigned long)k60_erase_buf;
	if (fsize > sizeof(k60_erase_buf))
		err |= 1;
	else
		memcpy((void *)to,(void *)from,fsize);
	fbnk->flerase = (int(*)())(to|1);

	/* WRITE: */
	from = (unsigned long)(k60_write) - 1;
	fsize = (unsigned long)(k60_write_end) - (unsigned long)(k60_write);
	to = (unsigned long)k60_write_buf;
	if (fsize > sizeof(k60_write_buf))
		err |= 2;
	else
		memcpy((void *)to,(void *)from,fsize);
	fbnk->flwrite = (int(*)())(to|1);

#if INCLUDE_FLASHEWRITE
	/* EWRITE: */
	from = (unsigned long)(k60_ewrite) - 1;
	fsize = (unsigned long)(k60_ewrite_end) - (unsigned long)(k60_ewrite);
	to = (unsigned long)k60_ewrite_buf;
	if (fsize > sizeof(k60_ewrite_buf))
		err |= 4;
	else
		memcpy((void *)to,(void *)from,fsize);
	fbnk->flewrite = (int(*)())(to|1);
#else
	fbnk->flewrite = k60_ewrite;
#endif

	if (err != 0) {
		printf("FlashInitErr: %d\n",err);
		return(-1);
	}

	fbnk->fltype = k60_type;
	fbnk->fllock = k60_lock;
	fbnk->sectors = sinfoK60;
	snum += FlashBankInit(fbnk,snum);

	return(0);
}
#endif
