#define ALLOCSIZE 		(8*1024)
#define MONSTACKSIZE	(4*1024)

/* One of four different TFS defrag modes:
 * Only 1 should be enabled; each has its pros/cons...
 *
 * DEFRAGTYPE_PWRSAFE:
 *   Provides powersafe defragmentation using spare sector.
 * DEFRAGTYPE_NPWRSAFE_SPARE:
 *   Not powersafe and uses spare (saves RAM).
 * DEFRAGTYPE_NPWRSAFE_APPRAMBASE:
 *   Not powersafe, assumes there is at least enough ram starting at
 *   APPRAMBASE to copy the largest sector to (saves life of sector,
 *   but assumes the ram will be available when needed).
 * DEFRAGTYPE_NPWRSAFE_RAMBLOCK:
 *   Not powersafe, allocates a static array in RAM that is the size
 *	 of the largest sector.
 */ 
#define DEFRAGTYPE_PWRSAFE				0
#define DEFRAGTYPE_NPWRSAFE_SPARE		0
#define DEFRAGTYPE_NPWRSAFE_APPRAMBASE	1
#define DEFRAGTYPE_NPWRSAFE_RAMBLOCK	0

#define FLASH_PROTECT_RANGE  	"0-31"

/* TFS definitions:
 * Values that configure the flash space that is allocated to TFS.
 * Fill in port specific values here.
 * Note that if we're using tfsclean4.c (simpler, not powersafe),
 * and we'er
 */
#if (DEFRAGTYPE_PWRSAFE | DEFRAGTYPE_NPWRSAFE_SPARE)
#define TFSSPARESIZE    		PSECTOR_SIZE
#define TFSSTART        		0x0007c000
#define TFSEND          		0x0007f7ff
#define TFSSPARE        		(TFSEND+1)
#define TFSSECTORCOUNT			7	// does not include SPARE
#else
#define TFSSTART        		0x0007c000
#define TFSEND          		0x0007ffff
#define TFSSECTORCOUNT			8
#define TFSSPARE        		0	// not used
#define TFSSPARESIZE    		0	// not used
#endif

/* INCLUDE_XXX Macros:
 * Set/clear the appropriate macros depending on what you want
 * to configure in your system.
 * Note that this is a reasonably "loaded" configuration.  For
 * a real system, several of these facilities are likely to be
 * off...
 */
								
#define INCLUDE_MALLOC			1
#define INCLUDE_MEMCMDS         0
#define INCLUDE_DM              1
#define INCLUDE_PM              1
#define INCLUDE_SHELLVARS		1
#define INCLUDE_XMODEM          0
#define INCLUDE_UNZIP           0
#define INCLUDE_ETHERNET        1
#define INCLUDE_ICMP			INCLUDE_ETHERNET
#define INCLUDE_TFTP            INCLUDE_ETHERNET
#define INCLUDE_FLASH           1
#define INCLUDE_TFS             1
#define INCLUDE_TFSAPI          INCLUDE_TFS
#define INCLUDE_TFSSCRIPT       INCLUDE_TFS
#define INCLUDE_TFSCLI          INCLUDE_TFS
#define INCLUDE_EDIT            INCLUDE_TFS
#define INCLUDE_LINEEDIT        1
#define INCLUDE_DHCPBOOT        INCLUDE_ETHERNET
#define INCLUDE_IPV6			0
#define INCLUDE_EE              0
#define INCLUDE_STOREMAC        0
#define INCLUDE_VERBOSEHELP     0
#define INCLUDE_HWTMR	 	    1
#define INCLUDE_BLINKLED		0
#define INCLUDE_ETHERCMD		1
#define INCLUDE_ICMPCMD			1
#define INCLUDE_FLASHEWRITE		INCLUDE_FLASH

/* Some fine tuning (if needed)...
 * If these #defines are not in config.h, they default to '1' in
 * various other include files within uMon source; hence, they're
 * really only useful if you need to turn off ('0') the specific
 * facility or block of code.
 */
#define INCLUDE_TFTPSRVR		INCLUDE_ETHERNET
#define INCLUDE_ETHERVERBOSE	1
#define INCLUDE_MONCMD			INCLUDE_ETHERNET
#define INCLUDE_READ			1
#define INCLUDE_ETHERCMD		1
#define INCLUDE_MONLIB			1
#define INCLUDE_FLASHENV		0

#include "inc_check.h"
